#include <stdlib.h>
#include <math.h>
#include "Noise1.h"

double catmull_rom(double x0, double x1, double x2, double x3, double t)
{
  double v1 = (x2 - x0) * 0.5, v2 = (x3 - x1) * 0.5;

  return (((2.0 * x1 - 2.0 * x2 + v1 + v2) * t
    - 3.0 * x1 + 3.0 * x2 - 2.0 * v1 - v2) * t
    + v1) * t + x1;
}

void Noise1::copy(const Noise1 &noise)
{
  n = noise.n;
  p = new double[n];

  for (int i = 0; i < n; ++i)
    p[i] = noise.p[i];
}

Noise1::Noise1()
{
  p = 0;
}

Noise1::Noise1(int n)
{
  this->n = n;
  p = new double[n];

  for (int i = 0; i < n; ++i)
    p[i] = (double)rand() / (double)RAND_MAX - 0.5;
}

Noise1::Noise1(const Noise1 &noise)
{
  copy(noise);
}

Noise1::~Noise1()
{
  if (p) delete[] p;
}

Noise1 &Noise1::operator=(const Noise1 &noise)
{
  this->~Noise1();
  copy(noise);

  return *this;
}

double Noise1::point(int i)
{
  return p[i];
}

double Noise1::noise(double x)
{
  double nx = (double)n * (x - floor(x));
  int i = (int)nx;
  double t = nx - (double)i;

  if (i == 0) {
    return catmull_rom(p[n - 1], p[0], p[1], p[2], t);
  }
  else if (i < n - 2) {
    return catmull_rom(p[i - 1], p[i], p[i + 1], p[i + 2], t);
  }
  else if (i == n - 2) {
    return catmull_rom(p[n - 3], p[n - 2], p[n - 1], p[0], t);
  }
  else {
    return catmull_rom(p[n - 2], p[n - 1], p[0], p[1], t);
  }
}

double Noise1::perlin(double x, int o, double a)
{
  double v = noise(x);
  double w = a;

  while (--o > 0) {
    v += noise(x *= 2.0) * w;
    w *= a;
  }

  return v;
}

double Noise1::turbulence(double x, int o, double a)
{
  double v = fabs(noise(x));
  double w = a;

  while (--o > 0) {
    v += fabs(noise(x *= 2.0)) * w;
    w *= a;
  }

  return v;
}
