#include <math.h>
#include <memory.h>
#include "Noise2.h"

void Noise2::copy(const Noise2 &noise)
{
  m = noise.m;
  p = new Noise1 *[m];

  for (int i = 0; i < m; ++i) {
    p[i] = new Noise1;
    *p[i] = *noise.p[i];
  }
}

Noise2::Noise2()
{
  p = 0;
}

Noise2::Noise2(int n, int m)
{
  this->m = m;
  p = new Noise1 *[m];

  for (int i = 0; i < m; ++i)
    p[i] = new Noise1(n);
}

Noise2::Noise2(const Noise2 &noise)
{
  copy(noise);
}

Noise2::~Noise2()
{
  if (p) {
    for (int i = 0; i < m; ++i)
      delete p[i];
    delete[] p;
  }
}

Noise2 &Noise2::operator=(const Noise2 &noise)
{
  this->~Noise2();
  copy(noise);

  return *this;
}

double Noise2::point(int i, int j)
{
  return p[j]->point(i);
}

double Noise2::noise(double x, double y)
{
  double my = (double)m * (y - floor(y));
  int i = (int)my;
  double t = my - (double)i;

  if (i == 0) {
    return catmull_rom(p[m - 1]->noise(x), p[0]->noise(x),
      p[1]->noise(x), p[2]->noise(x), t);
  }
  else if (i < m - 2) {
    return catmull_rom(p[i - 1]->noise(x), p[i]->noise(x),
      p[i + 1]->noise(x), p[i + 2]->noise(x), t);
  }
  else if (i == m - 2) {
    return catmull_rom(p[m - 3]->noise(x), p[m - 2]->noise(x),
      p[m - 1]->noise(x), p[0]->noise(x), t);
  }
  else {
    return catmull_rom(p[m - 2]->noise(x), p[m - 1]->noise(x),
      p[0]->noise(x), p[1]->noise(x), t);
  }
}

double Noise2::perlin(double x, double y, int o, double a)
{
  double v = noise(x, y);
  double w = a;

  while (--o > 0) {
    v += noise(x *= 2.0, y *= 2.0) * w;
    w *= a;
  }

  return v;
}

double Noise2::turbulence(double x, double y, int o, double a)
{
  double v = fabs(noise(x, y));
  double w = a;

  while (--o > 0) {
    v += fabs(noise(x *= 2.0, y *= 2.0)) * w;
    w *= a;
  }

  return v;
}
